Method: PathExpander#process
- Defined in:
- lib/path_expander.rb
#process ⇒ Object
Top-level method processes args. It replaces args’ contents with a new array of flags to process and returns a list of files to process. Eg
PathExpander.new(ARGV).process.each do |f|
puts "./#{f}"
end
122 123 124 125 126 127 128 |
# File 'lib/path_expander.rb', line 122 def process files, flags = process_args args.replace process_flags flags files.uniq end |